/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.6
  * @date     2021-12-31
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"

/* TMR AutoReload Value: for 16 bit tmr 0x0000~0xFFFF */
#define TMRx_PR 100

/** @addtogroup UTILITIES_examples
  * @{
  */
uint16_t PWM_DataLength=0;	
tmr_output_config_type tmr_output_struct;	
uint16_t src_buffer[] = {10,20,30,40,50,60,70,80,90,10,20,30,40,50,60,70,80,90};

/** @addtogroup AT32_TMR_DMA_Update_PWM_Duty_Cycle_Dynamically Duty
  * @{
  */

void crm_configuration(void);
void gpio_configuration(void);
void tmr_configuration(void);
void dma_configuration(void);
void TMR4_DMA_Duty_Cycle(void);
	
/**
  * @brief  configure the tmr pins.
  * @param  none
  * @retval none
  */
void gpio_configuration(void)
{
  gpio_init_type gpio_init_struct;

  gpio_default_para_init(&gpio_init_struct);

	/* configure PB7 tmr4_ch2 as output*/
  gpio_init_struct.gpio_pins = GPIO_PINS_7;
  gpio_init_struct.gpio_mode = GPIO_MODE_MUX;
  gpio_init(GPIOB, &gpio_init_struct);
}

/**
  * @brief  configure the tmr parameters.
  * @param  none
  * @retval none
  */
void tmr_configuration(void)
{
  /* Init TMR4 */	
	tmr_base_init(TMR4, TMRx_PR-1, 0);
  tmr_cnt_dir_set(TMR4, TMR_COUNT_UP);

	/* TMR configuration as output mode */
  tmr_output_default_para_init(&tmr_output_struct);
  tmr_output_struct.oc_mode = TMR_OUTPUT_CONTROL_PWM_MODE_A;
  tmr_output_struct.oc_output_state = TRUE;
  tmr_output_struct.oc_polarity = TMR_OUTPUT_ACTIVE_LOW;
	
	/* TMR4 channel 2 configuration */	
  tmr_output_channel_config(TMR4, TMR_SELECT_CHANNEL_2, &tmr_output_struct);

	/* enable tmr output channel buffer */
	tmr_output_channel_buffer_enable(TMR4, TMR_SELECT_CHANNEL_2, TRUE);
}


/**
  * @brief  configure the dma parameters.
  * @param  none
  * @retval none
  */
void dma_configuration(void)
{
	dma_init_type dma_init_struct = {0};
	uint16_t index = 0;
	
	while(index < PWM_DataLength)
	{
		src_buffer[index] = (uint16_t) (((uint32_t) (src_buffer[index]) * (TMRx_PR)) / (100));
		index++;
	}
	
  /* dma1 channel7 configuration */
  dma_default_para_init(&dma_init_struct);
	
  dma_init_struct.buffer_size = PWM_DataLength;
  dma_init_struct.direction = DMA_DIR_MEMORY_TO_PERIPHERAL;
  dma_init_struct.memory_base_addr = (uint32_t)src_buffer;
  dma_init_struct.memory_data_width = DMA_MEMORY_DATA_WIDTH_HALFWORD;
  dma_init_struct.memory_inc_enable = TRUE;
  dma_init_struct.peripheral_base_addr = (uint32_t)&TMR4->c2dt;
  dma_init_struct.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_HALFWORD;
  dma_init_struct.peripheral_inc_enable = FALSE;
  dma_init_struct.priority = DMA_PRIORITY_HIGH;
  dma_init_struct.loop_mode_enable = FALSE;
  dma_init(DMA1_CHANNEL7, &dma_init_struct);
}

/**
  * @brief  configures the different system clocks.
  * @param  none
  * @retval none
  */
void crm_configuration(void)
{
  /* tmr clock enable */
  crm_periph_clock_enable(CRM_TMR4_PERIPH_CLOCK, TRUE);
	
  /* gpio clock enable */
  crm_periph_clock_enable(CRM_GPIOB_PERIPH_CLOCK, TRUE);
	
	/* dma clock enable */
  crm_periph_clock_enable(CRM_DMA1_PERIPH_CLOCK, TRUE);
}

/**
  * @brief  configures the PWM dma duty cycle parameters.
  * @param  none
  * @retval none
  */
void TMR4_DMA_Duty_Cycle(void)
{

	/* set tmr channel CC value */	
	tmr_channel_value_set(TMR4, TMR_SELECT_CHANNEL_2, 0);

  /* config set the number of data to be transferred by dma */  
  dma_channel_enable(DMA1_CHANNEL7, FALSE);
	dma_data_number_set(DMA1_CHANNEL7, PWM_DataLength);
  dma_channel_enable(DMA1_CHANNEL7, TRUE);

	/* TMR enable counter */
	tmr_counter_enable(TMR4, TRUE);	
	
	/* wait for the end of dma transfer */  
	while (!dma_flag_get(DMA1_FDT7_FLAG)); 
	/* Clear dma flag */
	dma_flag_clear(DMA1_FDT7_FLAG);

	/* Clear TMR4 update Interrupt  pending bit */
	tmr_flag_clear(TMR4, TMR_OVF_FLAG);
  while(SET!=tmr_flag_get(TMR4, TMR_OVF_FLAG));
	/* Clear TMR4 update Interrupt  pending bit */
	tmr_flag_clear(TMR4, TMR_OVF_FLAG);
  while(SET!=tmr_flag_get(TMR4, TMR_OVF_FLAG));

	/* TMR disable counter */
	tmr_counter_enable(TMR4, FALSE);		

}

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
	/* the Number of Pwm Output */
	PWM_DataLength = sizeof(src_buffer)/sizeof(src_buffer[0]);
  /* system clocks configuration */
  system_clock_config();
		
	/*board initialize interface init led and button	*/
	at32_board_init();

	/* turn led2/led3/led4 on */
  at32_led_on(LED2);
  at32_led_on(LED3);
  at32_led_on(LED4);
	
	/* enable tmr/gpio clock */
	crm_configuration();
	
  /* tmr gpio configuration */
  gpio_configuration();

	/* dma configuration */
  dma_configuration();
	
	/* tmr configuration */
  tmr_configuration();

	/* enable tmr4 overflow dma request */
	tmr_dma_request_enable(TMR4, TMR_OVERFLOW_DMA_REQUEST, TRUE);
	
  while(1)
  {
		TMR4_DMA_Duty_Cycle();
		delay_us(10);		
	}
}

/**
  * @}
  */ 

/**
  * @}
  */ 
